@@ -1,21 +1,19 @@
 import asyncio
-from collections import deque, namedtuple
 import itertools
 import logging
 import os
 import threading
-import weakref
 import warnings
+import weakref
+from collections import deque, namedtuple
 
 from tornado.concurrent import Future
 from tornado.ioloop import IOLoop
 
 from ..protocol import nested_deserialize
 from ..utils import get_ip
-
+from .core import Comm, CommClosedError, Connector, Listener
 from .registry import Backend, backends
-from .core import Comm, Connector, Listener, CommClosedError
-
 
 logger = logging.getLogger(__name__)
 
@@ -45,7 +43,7 @@ class Manager:
     def add_listener(self, addr, listener):
         with self.lock:
             if addr in self.listeners:
-                raise RuntimeError("already listening on %r" % (addr,))
+                raise RuntimeError(f"already listening on {addr!r}")
             self.listeners[addr] = listener
 
     def remove_listener(self, addr):
@@ -155,12 +153,17 @@ class InProc(Comm):
     _initialized = False
 
     def __init__(
-        self, local_addr, peer_addr, read_q, write_q, write_loop, deserialize=True
+        self,
+        local_addr: str,
+        peer_addr: str,
+        read_q,
+        write_q,
+        write_loop,
+        deserialize: bool = True,
     ):
-        Comm.__init__(self)
+        super().__init__(deserialize=deserialize)
         self._local_addr = local_addr
         self._peer_addr = peer_addr
-        self.deserialize = deserialize
         self._read_q = read_q
         self._write_q = write_q
         self._write_loop = write_loop
@@ -172,28 +175,28 @@ class InProc(Comm):
 
     def _get_finalizer(self):
         def finalize(write_q=self._write_q, write_loop=self._write_loop, r=repr(self)):
-            logger.warning("Closing dangling queue in %s" % (r,))
+            logger.warning(f"Closing dangling queue in {r}")
             write_loop.add_callback(write_q.put_nowait, _EOF)
 
         return finalize
 
     @property
-    def local_address(self):
+    def local_address(self) -> str:
         return self._local_addr
 
     @property
-    def peer_address(self):
+    def peer_address(self) -> str:
         return self._peer_addr
 
     async def read(self, deserializers="ignored"):
         if self._closed:
-            raise CommClosedError
+            raise CommClosedError()
 
         msg = await self._read_q.get()
         if msg is _EOF:
             self._closed = True
             self._finalizer.detach()
-            raise CommClosedError
+            raise CommClosedError()
 
         if self.deserialize:
             msg = nested_deserialize(msg)
@@ -201,7 +204,7 @@ class InProc(Comm):
 
     async def write(self, msg, serializers=None, on_error=None):
         if self.closed():
-            raise CommClosedError
+            raise CommClosedError()
 
         # Ensure we feed the queue in the same thread it is read from.
         self._write_loop.add_callback(self._write_q.put_nowait, msg)
@@ -298,7 +301,7 @@ class InProcConnector(Connector):
     async def connect(self, address, deserialize=True, **connection_args):
         listener = self.manager.get_listener_for(address)
         if listener is None:
-            raise IOError("no endpoint for inproc address %r" % (address,))
+            raise OSError(f"no endpoint for inproc address {address!r}")
 
         conn_req = ConnectionRequest(
             c2s_q=Queue(),
