@@ -3,55 +3,30 @@ import gc
 import math
 from time import sleep
 
-import dask
 import pytest
 
-from distributed import Client, wait, Adaptive, LocalCluster, SpecCluster, Worker
-from distributed.utils_test import gen_test, slowinc, clean
-from distributed.utils_test import loop, nodebug, cleanup  # noqa: F401
+import dask
+
+from distributed import (
+    Adaptive,
+    Client,
+    LocalCluster,
+    Scheduler,
+    SpecCluster,
+    Worker,
+    wait,
+)
+from distributed.compatibility import LINUX, MACOS, WINDOWS
 from distributed.metrics import time
-
-
-@pytest.mark.asyncio
-async def test_simultaneous_scale_up_and_down(cleanup):
-    class TestAdaptive(Adaptive):
-        def get_scale_up_kwargs(self):
-            assert False
-
-        def _retire_workers(self):
-            assert False
-
-    class TestCluster(LocalCluster):
-        def scale_up(self, n, **kwargs):
-            assert False
-
-        def scale_down(self, workers):
-            assert False
-
-    with dask.config.set(
-        {"distributed.scheduler.default-task-durations": {"a": 4, "b": 4, "c": 1}}
-    ):
-        async with TestCluster(
-            n_workers=4, processes=False, asynchronous=True
-        ) as cluster:
-            async with Client(cluster, asynchronous=True) as c:
-                s = cluster.scheduler
-
-                future = c.map(slowinc, [1, 1, 1], key=["a-4", "b-4", "c-1"])
-
-                while len(s.rprocessing) < 3:
-                    await asyncio.sleep(0.001)
-
-                ta = cluster.adapt(
-                    interval="100 ms", scale_factor=2, Adaptive=TestAdaptive
-                )
-
-                await asyncio.sleep(0.3)
+from distributed.utils_test import async_wait_for, clean, gen_test, slowinc
 
 
 def test_adaptive_local_cluster(loop):
     with LocalCluster(
-        0, scheduler_port=0, silence_logs=False, dashboard_address=None, loop=loop
+        n_workers=0,
+        silence_logs=False,
+        dashboard_address=":0",
+        loop=loop,
     ) as cluster:
         alc = cluster.adapt(interval="100 ms")
         with Client(cluster, loop=loop) as c:
@@ -68,19 +43,18 @@ def test_adaptive_local_cluster(loop):
             start = time()
             while cluster.scheduler.nthreads:
                 sleep(0.01)
-                assert time() < start + 5
+                assert time() < start + 30
 
             assert not c.nthreads()
 
 
-@pytest.mark.asyncio
-async def test_adaptive_local_cluster_multi_workers(cleanup):
+@gen_test()
+async def test_adaptive_local_cluster_multi_workers():
     async with LocalCluster(
-        0,
-        scheduler_port=0,
+        n_workers=0,
         silence_logs=False,
         processes=False,
-        dashboard_address=None,
+        dashboard_address=":0",
         asynchronous=True,
     ) as cluster:
 
@@ -89,19 +63,14 @@ async def test_adaptive_local_cluster_multi_workers(cleanup):
         async with Client(cluster, asynchronous=True) as c:
             futures = c.map(slowinc, range(100), delay=0.01)
 
-            start = time()
             while not cluster.scheduler.workers:
                 await asyncio.sleep(0.01)
-                assert time() < start + 15, adapt.log
 
             await c.gather(futures)
             del futures
 
-            start = time()
-            # while cluster.workers:
             while cluster.scheduler.workers:
                 await asyncio.sleep(0.01)
-                assert time() < start + 15, adapt.log
 
             # no workers for a while
             for i in range(10):
@@ -113,12 +82,12 @@ async def test_adaptive_local_cluster_multi_workers(cleanup):
 
 
 @pytest.mark.xfail(reason="changed API")
-@pytest.mark.asyncio
-async def test_adaptive_scale_down_override(cleanup):
+@gen_test()
+async def test_adaptive_scale_down_override():
     class TestAdaptive(Adaptive):
         def __init__(self, *args, **kwargs):
             self.min_size = kwargs.pop("min_size", 0)
-            Adaptive.__init__(self, *args, **kwargs)
+            super().__init__(*args, **kwargs)
 
         async def workers_to_close(self, **kwargs):
             num_workers = len(self.cluster.workers)
@@ -132,7 +101,9 @@ async def test_adaptive_scale_down_override(cleanup):
         def scale_up(self, n, **kwargs):
             assert False
 
-    async with TestCluster(n_workers=10, processes=False, asynchronous=True) as cluster:
+    async with TestCluster(
+        n_workers=10, processes=False, asynchronous=True, dashboard_address=":0"
+    ) as cluster:
         ta = cluster.adapt(
             min_size=2, interval=0.1, scale_factor=2, Adaptive=TestAdaptive
         )
@@ -146,11 +117,10 @@ async def test_adaptive_scale_down_override(cleanup):
 @gen_test()
 async def test_min_max():
     cluster = await LocalCluster(
-        0,
-        scheduler_port=0,
+        n_workers=0,
         silence_logs=False,
         processes=False,
-        dashboard_address=None,
+        dashboard_address=":0",
         asynchronous=True,
         threads_per_worker=1,
     )
@@ -193,20 +163,19 @@ async def test_min_max():
         await cluster.close()
 
 
-@pytest.mark.asyncio
-async def test_avoid_churn(cleanup):
+@gen_test()
+async def test_avoid_churn():
     """We want to avoid creating and deleting workers frequently
 
     Instead we want to wait a few beats before removing a worker in case the
     user is taking a brief pause between work
     """
     async with LocalCluster(
-        0,
+        n_workers=0,
         asynchronous=True,
         processes=False,
-        scheduler_port=0,
         silence_logs=False,
-        dashboard_address=None,
+        dashboard_address=":0",
     ) as cluster:
         async with Client(cluster, asynchronous=True) as client:
             adapt = cluster.adapt(interval="20 ms", wait_count=5)
@@ -218,7 +187,7 @@ async def test_avoid_churn(cleanup):
             assert len(adapt.log) == 1
 
 
-@pytest.mark.asyncio
+@gen_test()
 async def test_adapt_quickly():
     """We want to avoid creating and deleting workers frequently
 
@@ -226,12 +195,11 @@ async def test_adapt_quickly():
     user is taking a brief pause between work
     """
     cluster = await LocalCluster(
-        0,
+        n_workers=0,
         asynchronous=True,
         processes=False,
-        scheduler_port=0,
         silence_logs=False,
-        dashboard_address=None,
+        dashboard_address=":0",
     )
     client = await Client(cluster, asynchronous=True)
     adapt = cluster.adapt(interval="20 ms", wait_count=5, maximum=10)
@@ -264,7 +232,6 @@ async def test_adapt_quickly():
 
         # Don't scale up for large sequential computations
         x = await client.scatter(1)
-        log = list(cluster._adaptive.log)
         for i in range(100):
             x = client.submit(slowinc, x)
 
@@ -275,16 +242,15 @@ async def test_adapt_quickly():
         await cluster.close()
 
 
-@gen_test(timeout=None)
+@gen_test()
 async def test_adapt_down():
-    """ Ensure that redefining adapt with a lower maximum removes workers """
+    """Ensure that redefining adapt with a lower maximum removes workers"""
     async with LocalCluster(
-        0,
+        n_workers=0,
         asynchronous=True,
         processes=False,
-        scheduler_port=0,
         silence_logs=False,
-        dashboard_address=None,
+        dashboard_address=":0",
     ) as cluster:
         async with Client(cluster, asynchronous=True) as client:
             cluster.adapt(interval="20ms", maximum=5)
@@ -298,20 +264,19 @@ async def test_adapt_down():
             start = time()
             while len(cluster.scheduler.workers) != 2:
                 await asyncio.sleep(0.1)
-                assert time() < start + 3
+                assert time() < start + 60
 
 
-@gen_test(timeout=30)
+@gen_test()
 async def test_no_more_workers_than_tasks():
     with dask.config.set(
         {"distributed.scheduler.default-task-durations": {"slowinc": 1000}}
     ):
         async with LocalCluster(
-            0,
-            scheduler_port=0,
+            n_workers=0,
             silence_logs=False,
             processes=False,
-            dashboard_address=None,
+            dashboard_address=":0",
             asynchronous=True,
         ) as cluster:
             adapt = cluster.adapt(minimum=0, maximum=4, interval="10 ms")
@@ -324,7 +289,7 @@ def test_basic_no_loop(loop):
     with clean(threads=False):
         try:
             with LocalCluster(
-                0, scheduler_port=0, silence_logs=False, dashboard_address=None
+                n_workers=0, silence_logs=False, dashboard_address=":0"
             ) as cluster:
                 with Client(cluster) as client:
                     cluster.adapt()
@@ -335,38 +300,35 @@ def test_basic_no_loop(loop):
             loop.add_callback(loop.stop)
 
 
-@pytest.mark.asyncio
+@pytest.mark.flaky(condition=LINUX, reruns=10, reruns_delay=5)
+@pytest.mark.xfail(condition=MACOS or WINDOWS, reason="extremely flaky")
+@gen_test()
 async def test_target_duration():
-    """ Ensure that redefining adapt with a lower maximum removes workers """
     with dask.config.set(
         {"distributed.scheduler.default-task-durations": {"slowinc": 1}}
     ):
         async with LocalCluster(
-            0,
+            n_workers=0,
             asynchronous=True,
             processes=False,
-            scheduler_port=0,
             silence_logs=False,
-            dashboard_address=None,
+            dashboard_address=":0",
         ) as cluster:
             adapt = cluster.adapt(interval="20ms", minimum=2, target_duration="5s")
             async with Client(cluster, asynchronous=True) as client:
-                while len(cluster.scheduler.workers) < 2:
-                    await asyncio.sleep(0.01)
-
+                await client.wait_for_workers(2)
                 futures = client.map(slowinc, range(100), delay=0.3)
+                await wait(futures)
 
-                while len(adapt.log) < 2:
-                    await asyncio.sleep(0.01)
-
-                assert adapt.log[0][1] == {"status": "up", "n": 2}
-                assert adapt.log[1][1] == {"status": "up", "n": 20}
+            assert adapt.log[0][1] == {"status": "up", "n": 2}
+            assert adapt.log[1][1] == {"status": "up", "n": 20}
 
 
-@pytest.mark.asyncio
-async def test_worker_keys(cleanup):
-    """ Ensure that redefining adapt with a lower maximum removes workers """
+@gen_test()
+async def test_worker_keys():
+    """Ensure that redefining adapt with a lower maximum removes workers"""
     async with SpecCluster(
+        scheduler={"cls": Scheduler, "options": {"dashboard_address": ":0"}},
         workers={
             "a-1": {"cls": Worker},
             "a-2": {"cls": Worker},
@@ -391,16 +353,15 @@ async def test_worker_keys(cleanup):
         assert names == {"a-1", "a-2"} or names == {"b-1", "b-2"}
 
 
-@pytest.mark.asyncio
-async def test_adapt_cores_memory(cleanup):
+@gen_test()
+async def test_adapt_cores_memory():
     async with LocalCluster(
-        0,
+        n_workers=0,
         threads_per_worker=2,
         memory_limit="3 GB",
-        scheduler_port=0,
         silence_logs=False,
         processes=False,
-        dashboard_address=None,
+        dashboard_address=":0",
         asynchronous=True,
     ) as cluster:
         adapt = cluster.adapt(minimum_cores=3, maximum_cores=9)
@@ -432,16 +393,15 @@ def test_adaptive_config():
         assert adapt.wait_count == 8
 
 
-@pytest.mark.asyncio
-async def test_update_adaptive(cleanup):
+@gen_test()
+async def test_update_adaptive():
     async with LocalCluster(
-        0,
+        n_workers=0,
         threads_per_worker=2,
         memory_limit="3 GB",
-        scheduler_port=0,
         silence_logs=False,
         processes=False,
-        dashboard_address=None,
+        dashboard_address=":0",
         asynchronous=True,
     ) as cluster:
         first = cluster.adapt(maxmimum=1)
@@ -449,3 +409,76 @@ async def test_update_adaptive(cleanup):
         await asyncio.sleep(0.2)
         assert first.periodic_callback is None
         assert second.periodic_callback.is_running()
+
+
+@gen_test()
+async def test_adaptive_no_memory_limit():
+    """Test that adapt() does not keep creating workers when no memory limit is set"""
+    async with LocalCluster(
+        n_workers=0,
+        threads_per_worker=1,
+        memory_limit=0,
+        asynchronous=True,
+        dashboard_address=":0",
+    ) as cluster:
+        cluster.adapt(minimum=1, maximum=10, interval="1 ms")
+        async with Client(cluster, asynchronous=True) as client:
+            await client.gather(client.map(slowinc, range(5), delay=0.35))
+        assert (
+            sum(
+                state[1]["n"]
+                for state in cluster._adaptive.log
+                if state[1]["status"] == "up"
+            )
+            <= 5
+        )
+
+
+@gen_test()
+async def test_scale_needs_to_be_awaited():
+    """
+    This tests that the adaptive class works fine if the scale method uses the
+    `sync` method to schedule its task instead of loop.add_callback
+    """
+
+    class RequiresAwaitCluster(LocalCluster):
+        def scale(self, n):
+            # super invocation in the nested function scope is messy
+            method = super().scale
+
+            async def _():
+                return method(n)
+
+            return self.sync(_)
+
+    async with RequiresAwaitCluster(
+        n_workers=0, asynchronous=True, dashboard_address=":0"
+    ) as cluster:
+        async with Client(cluster, asynchronous=True) as client:
+            futures = client.map(slowinc, range(5), delay=0.05)
+            assert len(cluster.workers) == 0
+            cluster.adapt()
+
+            await client.gather(futures)
+
+            del futures
+            await async_wait_for(lambda: not cluster.workers, 10)
+
+
+@gen_test()
+async def test_adaptive_stopped():
+    """
+    We should ensure that the adapt PC is actually stopped once the cluster
+    stops.
+    """
+    async with LocalCluster(
+        n_workers=0, asynchronous=True, dashboard_address=":0"
+    ) as cluster:
+        instance = cluster.adapt(interval="10ms")
+        assert instance.periodic_callback is not None
+
+        await async_wait_for(lambda: instance.periodic_callback.is_running(), timeout=5)
+
+        pc = instance.periodic_callback
+
+    await async_wait_for(lambda: not pc.is_running(), timeout=5)
