@@ -1,18 +1,22 @@
+from __future__ import annotations
+
+import filecmp
 import inspect
 import logging
 import os
 import shutil
 import sys
-from typing import List
-from types import ModuleType
-import filecmp
+import urllib.request
+from collections.abc import Iterable
 from importlib import import_module
+from types import ModuleType
+from typing import cast
 
 import click
-from tornado.httpclient import AsyncHTTPClient
 
 from dask.utils import tmpfile
 
+from .core import Server
 from .utils import import_file
 
 logger = logging.getLogger(__name__)
@@ -79,9 +83,9 @@ def _import_module(name, file_dir=None) -> ModuleType:
 
     Parameters
     ----------
-    name: str
+    name : str
         Module name, file path, or text of module or script
-    file_dir: string
+    file_dir : string
         Path of a directory where files should be copied
 
     Returns
@@ -119,13 +123,13 @@ def _import_module(name, file_dir=None) -> ModuleType:
     return module
 
 
-async def _download_module(url: str) -> ModuleType:
+def _download_module(url: str) -> ModuleType:
     logger.info("Downloading preload at %s", url)
     assert is_webaddress(url)
 
-    client = AsyncHTTPClient()
-    response = await client.fetch(url)
-    source = response.body.decode()
+    request = urllib.request.Request(url, method="GET")
+    response = urllib.request.urlopen(request)
+    source = response.read().decode()
 
     compiled = compile(source, url, "exec")
     module = ModuleType(url)
@@ -143,34 +147,39 @@ class Preload:
         The Worker or Scheduler
     name: str
         module name, file name, or web address to load
-    argv: [string]
+    argv: [str]
         List of string arguments passed to click-configurable `dask_setup`.
-    file_dir: string
+    file_dir: str
         Path of a directory where files should be copied
     """
 
-    def __init__(self, dask_server, name: str, argv: List[str], file_dir: str):
+    dask_server: Server
+    name: str
+    argv: list[str]
+    file_dir: str | None
+    module: ModuleType
+
+    def __init__(
+        self, dask_server: Server, name: str, argv: Iterable[str], file_dir: str | None
+    ):
         self.dask_server = dask_server
         self.name = name
-        self.argv = argv
+        self.argv = list(argv)
         self.file_dir = file_dir
 
-        if not is_webaddress(name):
-            self.module = _import_module(name, file_dir)
+        if is_webaddress(name):
+            self.module = _download_module(name)
         else:
-            self.module = None
+            self.module = _import_module(name, file_dir)
 
     async def start(self):
-        """ Run when the server finishes its start method """
-        if is_webaddress(self.name):
-            self.module = await _download_module(self.name)
-
+        """Run when the server finishes its start method"""
         dask_setup = getattr(self.module, "dask_setup", None)
 
         if dask_setup:
             if isinstance(dask_setup, click.Command):
                 context = dask_setup.make_context(
-                    "dask_setup", list(self.argv), allow_extra_args=False
+                    "dask_setup", self.argv, allow_extra_args=False
                 )
                 result = dask_setup.callback(
                     self.dask_server, *context.args, **context.params
@@ -185,7 +194,7 @@ class Preload:
                 logger.info("Run preload setup function: %s", self.name)
 
     async def teardown(self):
-        """ Run when the server starts its close method """
+        """Run when the server starts its close method"""
         dask_teardown = getattr(self.module, "dask_teardown", None)
         if dask_teardown:
             future = dask_teardown(self.dask_server)
@@ -194,9 +203,25 @@ class Preload:
 
 
 def process_preloads(
-    dask_server, preload: List[str], preload_argv: List[List], file_dir: str = None
-) -> List[Preload]:
+    dask_server,
+    preload: str | list[str],
+    preload_argv: list[str] | list[list[str]],
+    *,
+    file_dir: str | None = None,
+) -> list[Preload]:
     if isinstance(preload, str):
         preload = [preload]
+    if preload_argv and isinstance(preload_argv[0], str):
+        preload_argv = [cast("list[str]", preload_argv)] * len(preload)
+    elif not preload_argv:
+        preload_argv = [cast("list[str]", [])] * len(preload)
+    if len(preload) != len(preload_argv):
+        raise ValueError(
+            "preload and preload_argv have mismatched lengths "
+            f"{len(preload)} != {len(preload_argv)}"
+        )
 
-    return [Preload(dask_server, p, preload_argv, file_dir) for p in preload]
+    return [
+        Preload(dask_server, p, argv, file_dir)
+        for p, argv in zip(preload, preload_argv)
+    ]
