@@ -1,13 +1,14 @@
-import numpy as np
-from numpy.testing import assert_allclose
 import pytest
 
+np = pytest.importorskip("numpy")
 sparse = pytest.importorskip("sparse")
 
 from distributed.protocol import deserialize, serialize
 
 
 def test_serialize_deserialize_sparse():
+    from numpy.testing import assert_allclose
+
     x = np.random.random((2, 3, 4, 5))
     x[x < 0.8] = 0
 
