@@ -1,54 +1,46 @@
 import asyncio
 import os
 import socket
-import sys
 import threading
 import weakref
-import warnings
 
 import pytest
 
 import dask
+
+from distributed.comm.core import CommClosedError
 from distributed.core import (
-    pingpong,
+    ConnectionPool,
     Server,
     Status,
-    rpc,
-    connect,
-    send_recv,
+    clean_exception,
     coerce_to_address,
-    ConnectionPool,
+    connect,
+    pingpong,
+    rpc,
+    send_recv,
 )
-from distributed.protocol.compression import compressions
-
 from distributed.metrics import time
 from distributed.protocol import to_serialize
+from distributed.protocol.compression import compressions
 from distributed.utils import get_ip, get_ipv6
 from distributed.utils_test import (
-    gen_cluster,
-    has_ipv6,
     assert_can_connect,
-    assert_cannot_connect,
     assert_can_connect_from_everywhere_4,
     assert_can_connect_from_everywhere_4_6,
     assert_can_connect_from_everywhere_6,
     assert_can_connect_locally_4,
     assert_can_connect_locally_6,
-    tls_security,
+    assert_cannot_connect,
+    async_wait_for,
     captured_logger,
+    gen_cluster,
+    gen_test,
+    has_ipv6,
     inc,
     throws,
+    tls_security,
 )
-from distributed.utils_test import loop  # noqa F401
-
-
-EXTERNAL_IP4 = get_ip()
-if has_ipv6():
-    EXTERNAL_IP6 = get_ipv6()
-
-
-def echo(comm, x):
-    return x
 
 
 class CountedObject:
@@ -79,39 +71,13 @@ def echo_no_serialize(comm, x):
 
 
 def test_server_status_is_always_enum():
-    """
-    Assignments with strings get converted to corresponding Enum variant
-    """
+    """Assignments with strings is forbidden"""
     server = Server({})
     assert isinstance(server.status, Status)
-    with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter("ignore")
-        assert server.status != Status.stopped
-        server.status = "stopped"
-    assert isinstance(server.status, Status)
+    assert server.status != Status.stopped
+    server.status = Status.stopped
     assert server.status == Status.stopped
-
-
-def test_server_status_assign_non_variant_raises():
-    server = Server({})
-    with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter("ignore")
-        with pytest.raises(AssertionError):
-            server.status = "I do not exists"
-
-
-def test_server_status_assign_with_variant_warns():
-    server = Server({})
-    with warnings.catch_warnings(record=True) as w:
-        warnings.simplefilter("default")
-        with pytest.warns(PendingDeprecationWarning):
-            server.status = "running"
-
-
-def test_server_status_assign_with_variant_raises_in_tests():
-    """That would be the default in user code"""
-    server = Server({})
-    with pytest.raises(PendingDeprecationWarning):
+    with pytest.raises(TypeError):
         server.status = "running"
 
 
@@ -171,9 +137,9 @@ def test_server_raises_on_blocked_handlers(loop):
         await comm.write({"op": "ping"})
         msg = await comm.read()
 
-        assert "exception" in msg
-        assert isinstance(msg["exception"], ValueError)
-        assert "'ping' handler has been explicitly disallowed" in repr(msg["exception"])
+        _, exception, _ = clean_exception(msg["exception"])
+        assert isinstance(exception, ValueError)
+        assert "'ping' handler has been explicitly disallowed" in repr(exception)
 
         await comm.close()
         server.stop()
@@ -185,15 +151,20 @@ class MyServer(Server):
     default_port = 8756
 
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 7),
-    reason="asynccontextmanager not avaiable before Python 3.7",
-)
 @pytest.mark.asyncio
 async def test_server_listen():
     """
     Test various Server.listen() arguments and their effect.
     """
+    import socket
+
+    try:
+        EXTERNAL_IP4 = get_ip()
+        if has_ipv6():
+            EXTERNAL_IP6 = get_ipv6()
+    except socket.gaierror:
+        raise pytest.skip("no network access")
+
     from contextlib import asynccontextmanager
 
     @asynccontextmanager
@@ -442,17 +413,17 @@ async def test_rpc_with_many_connections_inproc():
 
 
 async def check_large_packets(listen_arg):
-    """ tornado has a 100MB cap by default """
-    server = Server({"echo": echo})
+    """tornado has a 100MB cap by default"""
+    server = Server({})
     await server.listen(listen_arg)
 
     data = b"0" * int(200e6)  # slightly more than 100MB
     async with rpc(server.address) as conn:
-        result = await conn.echo(x=data)
+        result = await conn.echo(data=data)
         assert result == data
 
         d = {"x": data}
-        result = await conn.echo(x=d)
+        result = await conn.echo(data=d)
         assert result == d
 
     server.stop()
@@ -544,23 +515,45 @@ async def test_connect_raises():
 
 @pytest.mark.asyncio
 async def test_send_recv_args():
-    server = Server({"echo": echo})
+    server = Server({})
     await server.listen(0)
 
     comm = await connect(server.address)
-    result = await send_recv(comm, op="echo", x=b"1")
+    result = await send_recv(comm, op="echo", data=b"1")
     assert result == b"1"
     assert not comm.closed()
-    result = await send_recv(comm, op="echo", x=b"2", reply=False)
+    result = await send_recv(comm, op="echo", data=b"2", reply=False)
     assert result is None
     assert not comm.closed()
-    result = await send_recv(comm, op="echo", x=b"3", close=True)
+    result = await send_recv(comm, op="echo", data=b"3", close=True)
     assert result == b"3"
     assert comm.closed()
 
     server.stop()
 
 
+@gen_test(timeout=5)
+async def test_send_recv_cancelled():
+    """Test that the comm channel is closed on CancelledError"""
+
+    async def get_stuck(comm):
+        await asyncio.Future()
+
+    server = Server({"get_stuck": get_stuck})
+    await server.listen(0)
+
+    client_comm = await connect(server.address, deserialize=False)
+    while not server._comms:
+        await asyncio.sleep(0.01)
+    server_comm = next(iter(server._comms))
+
+    with pytest.raises(asyncio.TimeoutError):
+        await asyncio.wait_for(send_recv(client_comm, op="get_stuck"), timeout=0.1)
+    assert client_comm.closed()
+    while not server_comm.closed():
+        await asyncio.sleep(0.01)
+
+
 def test_coerce_to_address():
     for arg in ["127.0.0.1:8786", ("127.0.0.1", 8786), ("127.0.0.1", "8786")]:
         assert coerce_to_address(arg) == "tcp://127.0.0.1:8786"
@@ -580,12 +573,12 @@ async def test_connection_pool():
 
     # Reuse connections
     await asyncio.gather(
-        *[rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[:5]]
+        *(rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[:5])
     )
-    await asyncio.gather(*[rpc(s.address).ping() for s in servers[:5]])
-    await asyncio.gather(*[rpc("127.0.0.1:%d" % s.port).ping() for s in servers[:5]])
+    await asyncio.gather(*(rpc(s.address).ping() for s in servers[:5]))
+    await asyncio.gather(*(rpc("127.0.0.1:%d" % s.port).ping() for s in servers[:5]))
     await asyncio.gather(
-        *[rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[:5]]
+        *(rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[:5])
     )
     assert sum(map(len, rpc.available.values())) == 5
     assert sum(map(len, rpc.occupied.values())) == 0
@@ -594,14 +587,14 @@ async def test_connection_pool():
 
     # Clear out connections to make room for more
     await asyncio.gather(
-        *[rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[5:]]
+        *(rpc(ip="127.0.0.1", port=s.port).ping() for s in servers[5:])
     )
     assert rpc.active == 0
     assert rpc.open == 5
 
     s = servers[0]
     await asyncio.gather(
-        *[rpc(ip="127.0.0.1", port=s.port).ping(delay=0.1) for i in range(3)]
+        *(rpc(ip="127.0.0.1", port=s.port).ping(delay=0.1) for i in range(3))
     )
     assert len(rpc.available["tcp://127.0.0.1:%d" % s.port]) == 3
 
@@ -615,6 +608,55 @@ async def test_connection_pool():
     await rpc.close()
 
 
+@pytest.mark.asyncio
+async def test_connection_pool_close_while_connecting(monkeypatch):
+    """
+    Ensure a closed connection pool guarantees to have no connections left open
+    even if it is closed mid-connecting
+    """
+    from distributed.comm.registry import backends
+    from distributed.comm.tcp import TCPBackend, TCPConnector
+
+    class SlowConnector(TCPConnector):
+        async def connect(self, address, deserialize, **connection_args):
+            await asyncio.sleep(0.1)
+            return await super().connect(
+                address, deserialize=deserialize, **connection_args
+            )
+
+    class SlowBackend(TCPBackend):
+        _connector_class = SlowConnector
+
+    monkeypatch.setitem(backends, "tcp", SlowBackend())
+
+    server = Server({})
+    await server.listen("tcp://")
+
+    pool = await ConnectionPool(limit=2)
+
+    async def connect_to_server():
+        comm = await pool.connect(server.address)
+        pool.reuse(server.address, comm)
+
+    tasks = [asyncio.create_task(connect_to_server()) for _ in range(30)]
+
+    await asyncio.sleep(0)
+    assert pool._connecting
+    close_fut = asyncio.create_task(pool.close())
+
+    with pytest.raises(
+        CommClosedError, match="ConnectionPool not running. Status: Status.closed"
+    ):
+        await asyncio.gather(*tasks)
+
+    await close_fut
+    assert not pool.open
+    assert not pool._n_connecting
+
+    for t in tasks:
+        t.cancel()
+
+
 @pytest.mark.asyncio
 async def test_connection_pool_respects_limit():
 
@@ -635,7 +677,7 @@ async def test_connection_pool_respects_limit():
 
     pool = await ConnectionPool(limit=limit)
 
-    await asyncio.gather(*[do_ping(pool, s.port) for s in servers])
+    await asyncio.gather(*(do_ping(pool, s.port) for s in servers))
 
 
 @pytest.mark.asyncio
@@ -657,9 +699,9 @@ async def test_connection_pool_tls():
 
     rpc = await ConnectionPool(limit=5, connection_args=connection_args)
 
-    await asyncio.gather(*[rpc(s.address).ping() for s in servers[:5]])
-    await asyncio.gather(*[rpc(s.address).ping() for s in servers[::2]])
-    await asyncio.gather(*[rpc(s.address).ping() for s in servers])
+    await asyncio.gather(*(rpc(s.address).ping() for s in servers[:5]))
+    await asyncio.gather(*(rpc(s.address).ping() for s in servers[::2]))
+    await asyncio.gather(*(rpc(s.address).ping() for s in servers))
     assert rpc.active == 0
 
     await rpc.close()
@@ -677,8 +719,8 @@ async def test_connection_pool_remove():
 
     rpc = await ConnectionPool(limit=10)
     serv = servers.pop()
-    await asyncio.gather(*[rpc(s.address).ping() for s in servers])
-    await asyncio.gather(*[rpc(serv.address).ping() for i in range(3)])
+    await asyncio.gather(*(rpc(s.address).ping() for s in servers))
+    await asyncio.gather(*(rpc(serv.address).ping() for i in range(3)))
     await rpc.connect(serv.address)
     assert sum(map(len, rpc.available.values())) == 6
     assert sum(map(len, rpc.occupied.values())) == 1
@@ -765,22 +807,20 @@ def test_compression(compression, serialize, loop):
         loop.run_sync(f)
 
 
-def test_rpc_serialization(loop):
-    async def f():
-        server = Server({"echo": echo_serialize})
-        await server.listen("tcp://")
+@pytest.mark.asyncio
+async def test_rpc_serialization():
+    server = Server({"echo": echo_serialize})
+    await server.listen("tcp://")
 
-        async with rpc(server.address, serializers=["msgpack"]) as r:
-            with pytest.raises(TypeError):
-                await r.echo(x=to_serialize(inc))
+    async with rpc(server.address, serializers=["msgpack"]) as r:
+        with pytest.raises(TypeError):
+            await r.echo(x=to_serialize(inc))
 
-        async with rpc(server.address, serializers=["msgpack", "pickle"]) as r:
-            result = await r.echo(x=to_serialize(inc))
-            assert result == {"result": inc}
+    async with rpc(server.address, serializers=["msgpack", "pickle"]) as r:
+        result = await r.echo(x=to_serialize(inc))
+        assert result == {"result": inc}
 
-        server.stop()
-
-    loop.run_sync(f)
+    server.stop()
 
 
 @gen_cluster()
@@ -832,3 +872,125 @@ async def test_connection_pool_detects_remote_close():
     # while creating conn2:
     p._validate()
     await p.close()
+
+
+@pytest.mark.asyncio
+async def test_close_properly():
+    """
+    If the server is closed we should cancel all still ongoing coros and close
+    all listeners.
+    GH4704
+    """
+
+    async def sleep(comm=None):
+        # We want to ensure this is actually canceled therefore don't give it a
+        # chance to actually complete
+        await asyncio.sleep(2000000)
+
+    server = await Server({"sleep": sleep})
+    assert server.status == Status.running
+    ports = [8881, 8882, 8883]
+
+    # Previously we close *one* listener, therefore ensure we always use more
+    # than one for this test
+    assert len(ports) > 1
+    for port in ports:
+        await server.listen(port)
+    # We use TCP here for simplicity. If they are closed we should expect other
+    # backends to close properly as well
+    ip = get_ip()
+    rpc_addr = f"tcp://{ip}:{ports[-1]}"
+    async with rpc(rpc_addr) as remote:
+
+        comm = await remote.live_comm()
+        await comm.write({"op": "sleep"})
+
+        await async_wait_for(lambda: not server._ongoing_coroutines, 10)
+
+        listeners = server.listeners
+        assert len(listeners) == len(ports)
+
+        for port in ports:
+            await assert_can_connect(f"tcp://{ip}:{port}")
+
+        await server.close()
+
+        for port in ports:
+            await assert_cannot_connect(f"tcp://{ip}:{port}")
+
+        # weakref set/dict should be cleaned up
+        assert not len(server._ongoing_coroutines)
+
+
+@pytest.mark.asyncio
+async def test_server_redundant_kwarg():
+    with pytest.raises(TypeError, match="unexpected keyword argument"):
+        await Server({}, typo_kwarg="foo")
+
+
+@pytest.mark.asyncio
+async def test_server_comms_mark_active_handlers():
+    """Whether handlers are active can be read off of the self._comms values.
+    ensure this is properly reflected and released. The sentinel for
+    "open comm but no active handler" is `None`
+    """
+
+    async def long_handler(comm):
+        await asyncio.sleep(0.2)
+        return "done"
+
+    server = await Server({"wait": long_handler})
+    await server.listen(0)
+    assert server._comms == {}
+
+    comm = await connect(server.address)
+    await comm.write({"op": "wait"})
+    while not server._comms:
+        await asyncio.sleep(0.05)
+    assert set(server._comms.values()) == {"wait"}
+    assert await comm.read() == "done"
+    assert set(server._comms.values()) == {None}
+    await comm.close()
+    while server._comms:
+        await asyncio.sleep(0.01)
+
+
+@pytest.mark.asyncio
+async def test_close_fast_without_active_handlers():
+    async def very_fast(comm):
+        return "done"
+
+    server = await Server({"do_stuff": very_fast})
+    await server.listen(0)
+    assert server._comms == {}
+
+    comm = await connect(server.address)
+    await comm.write({"op": "do_stuff"})
+    while not server._comms:
+        await asyncio.sleep(0.05)
+    fut = server.close()
+
+    await asyncio.wait_for(fut, 0.1)
+
+
+@pytest.mark.asyncio
+async def test_close_grace_period_for_handlers():
+    async def long_handler(comm, delay=10):
+        await asyncio.sleep(delay)
+        return "done"
+
+    server = await Server({"wait": long_handler})
+    await server.listen(0)
+    assert server._comms == {}
+
+    comm = await connect(server.address)
+    await comm.write({"op": "wait"})
+    while not server._comms:
+        await asyncio.sleep(0.05)
+    fut = server.close()
+    # since the handler is running for a while, the close will not immediately
+    # go through. We'll give the comm about a second to close itself
+    with pytest.raises(asyncio.TimeoutError):
+        await asyncio.wait_for(fut, 0.5)
+    await comm.close()
+    await server.close()
