@@ -1,19 +1,24 @@
-from __future__ import print_function, division, absolute_import
-
 import pytest
 
 from distributed.counter import Counter
-from distributed.utils_test import loop  # noqa F401
 
 try:
     from distributed.counter import Digest
 except ImportError:
-    Digest = None
+    Digest = None  # type: ignore
 
 
-@pytest.mark.parametrize('CD,size', [(Counter, lambda d: sum(d.values())),
-                                     pytest.mark.skipif(not Digest, reason="no crick library")(
-                                     (Digest, lambda x: x.size()))])
+@pytest.mark.parametrize(
+    "CD,size",
+    [
+        (Counter, lambda d: sum(d.values())),
+        pytest.param(
+            Digest,
+            lambda x: x.size(),
+            marks=pytest.mark.skipif(not Digest, reason="no crick library"),
+        ),
+    ],
+)
 def test_digest(loop, CD, size):
     c = CD(loop=loop)
     c.add(1)
