@@ -1,4 +1,5 @@
 import asyncio
+import contextlib
 import itertools
 import logging
 import random
@@ -7,9 +8,13 @@ import weakref
 from operator import mul
 from time import sleep
 
-import dask
 import pytest
-from distributed import Nanny, Worker, wait, worker_client
+from tlz import concat, sliding_window
+
+import dask
+
+from distributed import Lock, Nanny, Worker, wait, worker_client
+from distributed.compatibility import LINUX, WINDOWS
 from distributed.config import config
 from distributed.metrics import time
 from distributed.scheduler import key_split
@@ -24,17 +29,16 @@ from distributed.utils_test import (
     slowidentity,
     slowinc,
 )
-from tlz import concat, sliding_window
+
+pytestmark = pytest.mark.ci1
 
 # Most tests here are timing-dependent
 setup_module = nodebug_setup_module
 teardown_module = nodebug_teardown_module
 
 
-@pytest.mark.skipif(
-    not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
-)
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 2), ("127.0.0.2", 2)], timeout=20)
+@pytest.mark.skipif(not LINUX, reason="Need 127.0.0.2 to mean localhost")
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 2), ("127.0.0.2", 2)])
 async def test_work_stealing(c, s, a, b):
     [x] = await c._scatter([1], workers=a.address)
     futures = c.map(slowadd, range(50), [x] * 50)
@@ -73,7 +77,7 @@ async def test_steal_cheap_data_slow_computation(c, s, a, b):
     assert abs(len(a.data) - len(b.data)) <= 5
 
 
-@pytest.mark.avoid_travis
+@pytest.mark.avoid_ci
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
 async def test_steal_expensive_data_slow_computation(c, s, a, b):
     np = pytest.importorskip("numpy")
@@ -107,20 +111,136 @@ async def test_worksteal_many_thieves(c, s, *workers):
     assert sum(map(len, s.has_what.values())) < 150
 
 
-@pytest.mark.xfail(reason="GH#3574")
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-async def test_dont_steal_unknown_functions(c, s, a, b):
-    futures = c.map(inc, range(100), workers=a.address, allow_other_workers=True)
-    await wait(futures)
-    assert len(a.data) >= 95, [len(a.data), len(b.data)]
+@gen_cluster(
+    client=True,
+    nthreads=[("127.0.0.1", 1)] * 2,
+    config={"distributed.scheduler.work-stealing-interval": "10ms"},
+)
+async def test_stop_plugin(c, s, a, b):
+    steal = s.extensions["stealing"]
+
+    await steal.stop()
+    futs = c.map(slowinc, range(10), workers=[a.address], allow_other_workers=True)
+    await c.gather(futs)
+    assert len(a.data) == 10
+
+    # nothing happens
+    for _ in range(10):
+        await steal.stop()
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
+@gen_cluster(
+    client=True,
+    nthreads=[("127.0.0.1", 1)] * 2,
+    config={"distributed.scheduler.work-stealing-interval": "1ms"},
+)
+async def test_stop_in_flight(c, s, a, b):
+    steal = s.extensions["stealing"]
+    num_tasks = 10
+    futs = c.map(
+        slowinc, range(num_tasks), workers=[a.address], allow_other_workers=True
+    )
+    while not steal.in_flight:
+        await asyncio.sleep(0)
+
+    assert steal.in_flight
+    await steal.stop()
+    assert not steal.in_flight
+
+    assert len(a.data) != num_tasks
+    del futs
+    while s.tasks or a.tasks or b.tasks:
+        await asyncio.sleep(0.1)
+
+    futs = c.map(
+        slowinc, range(num_tasks), workers=[a.address], allow_other_workers=True
+    )
+    await c.gather(futs)
+    assert len(a.data) == num_tasks
+
+    del futs
+    while s.tasks or a.tasks or b.tasks:
+        await asyncio.sleep(0.1)
+    futs = c.map(
+        slowinc, range(num_tasks), workers=[a.address], allow_other_workers=True
+    )
+    while not len(a.tasks) == num_tasks:
+        await asyncio.sleep(0.01)
+    assert len(b.tasks) == 0
+    await steal.start()
+    await c.gather(futs)
+    assert len(a.tasks) != num_tasks
+    assert len(b.tasks) != 0
+
+
+@gen_cluster(
+    client=True,
+    nthreads=[("127.0.0.1", 1)] * 2,
+    config={"distributed.scheduler.work-stealing-interval": "10ms"},
+)
+async def test_allow_tasks_stolen_before_first_completes(c, s, a, b):
+    # https://github.com/dask/distributed/issues/5564
+    from distributed import Semaphore
+
+    steal = s.extensions["stealing"]
+    await steal.stop()
+    lock = await Semaphore(max_leases=1)
+
+    # We will reuse the same function such that multiple dispatches have the
+    # same task prefix. This ensures that we have tasks queued up but all of
+    # them are still classified as unknown.
+    # The lock allows us to control the duration of the first task without
+    # delaying test runtime or flakyness
+    def blocked_task(x, lock):
+        if x == 0:
+            with lock:
+                return x
+        return x
+
+    async with lock:
+        first = c.submit(blocked_task, 0, lock, workers=[a.address], key="f-0")
+        while first.key not in a.tasks:
+            await asyncio.sleep(0.001)
+        # Ensure the task is indeed blocked
+        with pytest.raises(asyncio.TimeoutError):
+            await asyncio.wait_for(first, 0.01)
+
+        more_tasks = c.map(
+            blocked_task,
+            # zero is a sentinel for using the lock.
+            # Start counting at one for non-blocking funcs
+            range(1, 11),
+            lock=lock,
+            workers=[a.address],
+            key=[f"f-{ix}" for ix in range(1, 11)],
+            allow_other_workers=True,
+        )
+        # All tasks are put on A since this is what we asked for. Only work
+        # stealing should rebalance the tasks once we allow for it
+        while not len(a.tasks) == 11:
+            await asyncio.sleep(0.1)
+
+        assert len(b.tasks) == 0
+
+        await steal.start()
+        # A is still blocked by executing task f-1 so this can only pass if
+        # workstealing moves the tasks to B
+        await c.gather(more_tasks)
+        assert len(b.data) == 10
+    await first
+
+
+@gen_cluster(
+    client=True,
+    nthreads=[("127.0.0.1", 1)] * 2,
+    config={"distributed.scheduler.work-stealing-interval": "10ms"},
+)
 async def test_eventually_steal_unknown_functions(c, s, a, b):
     futures = c.map(
         slowinc, range(10), delay=0.1, workers=a.address, allow_other_workers=True
     )
     await wait(futures)
+    assert not s.unknown_durations
     assert len(a.data) >= 3, [len(a.data), len(b.data)]
     assert len(b.data) >= 3, [len(a.data), len(b.data)]
 
@@ -142,53 +262,50 @@ async def test_steal_related_tasks(e, s, a, b, c):
     assert nearby > 10
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 10, timeout=1000)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 10)
 async def test_dont_steal_fast_tasks_compute_time(c, s, *workers):
-    np = pytest.importorskip("numpy")
-    x = c.submit(np.random.random, 10000000, workers=workers[0].address)
-
     def do_nothing(x, y=None):
         pass
 
-    # execute and measure runtime once
-    await wait(c.submit(do_nothing, 1))
+    xs = c.map(do_nothing, range(10), workers=workers[0].address)
+    await wait(xs)
 
-    futures = c.map(do_nothing, range(1000), y=x)
+    futures = c.map(do_nothing, range(100), y=xs)
 
     await wait(futures)
 
-    assert len(s.who_has[x.key]) == 1
-    assert len(s.has_what[workers[0].address]) == 1001
+    assert len(set.union(*(s.who_has[x.key] for x in xs))) == 1
+    assert len(s.has_what[workers[0].address]) == len(xs) + len(futures)
 
 
 @gen_cluster(client=True)
-async def test_dont_steal_fast_tasks_blacklist(c, s, a, b):
+async def test_dont_steal_fast_tasks_blocklist(c, s, a, b):
     # create a dependency
     x = c.submit(slowinc, 1, workers=[b.address])
 
-    # If the blacklist of fast tasks is tracked somewhere else, this needs to be
-    # changed. This test requires *any* key which is blacklisted.
+    # If the blocklist of fast tasks is tracked somewhere else, this needs to be
+    # changed. This test requires *any* key which is blocked.
     from distributed.stealing import fast_tasks
 
-    blacklisted_key = next(iter(fast_tasks))
+    blocked_key = next(iter(fast_tasks))
 
-    def fast_blacklisted(x, y=None):
+    def fast_blocked(x, y=None):
         # The task should observe a certain computation time such that we can
-        # ensure that it is not stolen due to the blacklisting. If it is too
+        # ensure that it is not stolen due to the blocking. If it is too
         # fast, the standard mechanism shouldn't allow stealing
         import time
 
         time.sleep(0.01)
 
     futures = c.map(
-        fast_blacklisted,
+        fast_blocked,
         range(100),
         y=x,
         # Submit the task to one worker but allow it to be distributed else,
         # i.e. this is not a task restriction
         workers=[a.address],
         allow_other_workers=True,
-        key=blacklisted_key,
+        key=blocked_key,
     )
 
     await wait(futures)
@@ -198,7 +315,7 @@ async def test_dont_steal_fast_tasks_blacklist(c, s, a, b):
     assert len(s.has_what[b.address]) == 1
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)], timeout=20)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)])
 async def test_new_worker_steals(c, s, a):
     await wait(c.submit(slowinc, 1, delay=0.01))
 
@@ -220,7 +337,7 @@ async def test_new_worker_steals(c, s, a):
     await b.close()
 
 
-@gen_cluster(client=True, timeout=20)
+@gen_cluster(client=True)
 async def test_work_steal_no_kwargs(c, s, a, b):
     await wait(c.submit(slowinc, 1, delay=0.05))
 
@@ -252,7 +369,7 @@ async def test_dont_steal_worker_restrictions(c, s, a, b):
     assert len(a.tasks) == 100
     assert len(b.tasks) == 0
 
-    result = s.extensions["stealing"].balance()
+    s.extensions["stealing"].balance()
 
     await asyncio.sleep(0.1)
 
@@ -286,9 +403,7 @@ async def test_steal_worker_restrictions(c, s, wa, wb, wc):
     assert len(wc.tasks) == 0
 
 
-@pytest.mark.skipif(
-    not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
-)
+@pytest.mark.skipif(not LINUX, reason="Need 127.0.0.2 to mean localhost")
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1), ("127.0.0.2", 1)])
 async def test_dont_steal_host_restrictions(c, s, a, b):
     future = c.submit(slowinc, 1, delay=0.10, workers=a.address)
@@ -307,9 +422,7 @@ async def test_dont_steal_host_restrictions(c, s, a, b):
     assert len(b.tasks) == 0
 
 
-@pytest.mark.skipif(
-    not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
-)
+@pytest.mark.skipif(not LINUX, reason="Need 127.0.0.2 to mean localhost")
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1), ("127.0.0.2", 2)])
 async def test_steal_host_restrictions(c, s, wa, wb):
     future = c.submit(slowinc, 1, delay=0.10, workers=wa.address)
@@ -355,9 +468,7 @@ async def test_dont_steal_resource_restrictions(c, s, a, b):
     assert len(b.tasks) == 0
 
 
-@gen_cluster(
-    client=True, nthreads=[("127.0.0.1", 1, {"resources": {"A": 2}})], timeout=3
-)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1, {"resources": {"A": 2}})])
 async def test_steal_resource_restrictions(c, s, a):
     future = c.submit(slowinc, 1, delay=0.10, workers=a.address)
     await future
@@ -369,10 +480,8 @@ async def test_steal_resource_restrictions(c, s, a):
 
     b = await Worker(s.address, loop=s.loop, nthreads=1, resources={"A": 4})
 
-    start = time()
     while not b.tasks or len(a.tasks) == 101:
         await asyncio.sleep(0.01)
-        assert time() < start + 3
 
     assert len(b.tasks) > 0
     assert len(a.tasks) < 101
@@ -380,10 +489,42 @@ async def test_steal_resource_restrictions(c, s, a):
     await b.close()
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 5, timeout=20)
-async def test_balance_without_dependencies(c, s, *workers):
-    s.extensions["stealing"]._pc.callback_time = 20
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1, {"resources": {"A": 2, "C": 1}})])
+async def test_steal_resource_restrictions_asym_diff(c, s, a):
+    # See https://github.com/dask/distributed/issues/5565
+    future = c.submit(slowinc, 1, delay=0.10, workers=a.address)
+    await future
 
+    futures = c.map(slowinc, range(100), delay=0.2, resources={"A": 1})
+    while len(a.tasks) < 101:
+        await asyncio.sleep(0.01)
+    assert len(a.tasks) == 101
+
+    b = await Worker(
+        s.address,
+        loop=s.loop,
+        nthreads=1,
+        resources={
+            "A": 4,
+            "B": 5,
+        },
+    )
+
+    while not b.tasks or len(a.tasks) == 101:
+        await asyncio.sleep(0.01)
+
+    assert len(b.tasks) > 0
+    assert len(a.tasks) < 101
+
+    await b.close()
+
+
+@gen_cluster(
+    client=True,
+    nthreads=[("127.0.0.1", 1)] * 5,
+    config={"distributed.scheduler.work-stealing-interval": "20ms"},
+)
+async def test_balance_without_dependencies(c, s, *workers):
     def slow(x):
         y = random.random() * 0.1
         sleep(y)
@@ -407,13 +548,31 @@ async def test_dont_steal_executing_tasks(c, s, a, b):
     assert len(b.data) == 0
 
 
+@gen_cluster(client=True)
+async def test_dont_steal_executing_tasks_2(c, s, a, b):
+    steal = s.extensions["stealing"]
+
+    future = c.submit(slowinc, 1, delay=0.5, workers=a.address)
+    while not a.executing_count:
+        await asyncio.sleep(0.01)
+
+    steal.move_task_request(
+        s.tasks[future.key], s.workers[a.address], s.workers[b.address]
+    )
+    await asyncio.sleep(0.1)
+    assert a.tasks[future.key].state == "executing"
+    assert not b.executing_count
+
+
 @gen_cluster(
     client=True,
     nthreads=[("127.0.0.1", 1)] * 10,
-    config={"distributed.scheduler.default-task-durations": {"slowidentity": 0.2}},
+    config={
+        "distributed.scheduler.default-task-durations": {"slowidentity": 0.2},
+        "distributed.scheduler.work-stealing-interval": "20ms",
+    },
 )
 async def test_dont_steal_few_saturated_tasks_many_workers(c, s, a, *rest):
-    s.extensions["stealing"]._pc.callback_time = 20
     x = c.submit(mul, b"0", 100000000, workers=a.address)  # 100 MB
     await wait(x)
 
@@ -429,10 +588,12 @@ async def test_dont_steal_few_saturated_tasks_many_workers(c, s, a, *rest):
     client=True,
     nthreads=[("127.0.0.1", 1)] * 10,
     worker_kwargs={"memory_limit": MEMORY_LIMIT},
-    config={"distributed.scheduler.default-task-durations": {"slowidentity": 0.2}},
+    config={
+        "distributed.scheduler.default-task-durations": {"slowidentity": 0.2},
+        "distributed.scheduler.work-stealing-interval": "20ms",
+    },
 )
 async def test_steal_when_more_tasks(c, s, a, *rest):
-    s.extensions["stealing"]._pc.callback_time = 20
     x = c.submit(mul, b"0", 50000000, workers=a.address)  # 50 MB
     await wait(x)
 
@@ -451,7 +612,8 @@ async def test_steal_when_more_tasks(c, s, a, *rest):
         "distributed.scheduler.default-task-durations": {
             "slowidentity": 0.2,
             "slow2": 1,
-        }
+        },
+        "distributed.scheduler.work-stealing-interval": "20ms",
     },
 )
 async def test_steal_more_attractive_tasks(c, s, a, *rest):
@@ -459,7 +621,6 @@ async def test_steal_more_attractive_tasks(c, s, a, *rest):
         sleep(1)
         return x
 
-    s.extensions["stealing"]._pc.callback_time = 20
     x = c.submit(mul, b"0", 100000000, workers=a.address)  # 100 MB
     await wait(x)
 
@@ -479,7 +640,7 @@ def func(x):
 
 async def assert_balanced(inp, expected, c, s, *workers):
     steal = s.extensions["stealing"]
-    steal._pc.stop()
+    await steal.stop()
 
     counter = itertools.count()
     tasks = list(concat(inp))
@@ -520,7 +681,7 @@ async def assert_balanced(inp, expected, c, s, *workers):
             await asyncio.sleep(0.001)
 
         result = [
-            sorted([int(key_split(k)) for k in s.processing[w.address]], reverse=True)
+            sorted((int(key_split(k)) for k in s.processing[w.address]), reverse=True)
             for w in workers
         ]
 
@@ -535,7 +696,7 @@ async def assert_balanced(inp, expected, c, s, *workers):
 
         if result2 == expected2:
             return
-    raise Exception("Expected: {}; got: {}".format(str(expected2), str(result2)))
+    raise Exception(f"Expected: {expected2}; got: {result2}")
 
 
 @pytest.mark.parametrize(
@@ -563,6 +724,7 @@ async def assert_balanced(inp, expected, c, s, *workers):
         pytest.param(
             [[1, 1, 1, 1, 1, 1, 1], [1, 1], [1, 1], [1, 1], []],
             [[1, 1, 1, 1, 1], [1, 1], [1, 1], [1, 1], [1, 1]],
+            # Can't mark as flaky as when it fails it does so every time for some reason
             marks=pytest.mark.xfail(
                 reason="Some uncertainty based on executing stolen task"
             ),
@@ -585,19 +747,23 @@ def test_balance(inp, expected):
     test()
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2, Worker=Nanny, timeout=20)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2, Worker=Nanny, timeout=60)
 async def test_restart(c, s, a, b):
     futures = c.map(
-        slowinc, range(100), delay=0.1, workers=a.address, allow_other_workers=True
+        slowinc, range(100), delay=0.01, workers=a.address, allow_other_workers=True
     )
     while not s.processing[b.worker_address]:
         await asyncio.sleep(0.01)
 
+    # Unknown tasks are never stolen therefore wait for a measurement
+    while not any(s.tasks[f.key].state == "memory" for f in futures):
+        await asyncio.sleep(0.01)
+
     steal = s.extensions["stealing"]
     assert any(st for st in steal.stealable_all)
     assert any(x for L in steal.stealable.values() for x in L)
 
-    await c.restart(timeout=10)
+    await c.restart()
 
     assert not any(x for x in steal.stealable_all)
     assert not any(x for L in steal.stealable.values() for x in L)
@@ -635,6 +801,12 @@ async def test_steal_communication_heavy_tasks(c, s, a, b):
     assert s.processing[b.address]
 
 
+@pytest.mark.flaky(
+    condition=WINDOWS and sys.version_info[:2] == (3, 7),
+    reruns=20,
+    reruns_delay=5,
+    reason="b.in_flight_tasks == 1",
+)
 @gen_cluster(client=True)
 async def test_steal_twice(c, s, a, b):
     x = c.submit(inc, 1, workers=a.address)
@@ -646,7 +818,7 @@ async def test_steal_twice(c, s, a, b):
         await asyncio.sleep(0.01)
 
     # Army of new workers arrives to help
-    workers = await asyncio.gather(*[Worker(s.address, loop=s.loop) for _ in range(20)])
+    workers = await asyncio.gather(*(Worker(s.address, loop=s.loop) for _ in range(20)))
 
     await wait(futures)
 
@@ -659,48 +831,38 @@ async def test_steal_twice(c, s, a, b):
         )
     assert max(map(len, has_what.values())) < 30
 
+    assert a.in_flight_tasks == 0
+    assert b.in_flight_tasks == 0
+
     await c._close()
-    await asyncio.gather(*[w.close() for w in workers])
-
-
-@gen_cluster(client=True)
-async def test_dont_steal_executing_tasks(c, s, a, b):
-    steal = s.extensions["stealing"]
-
-    future = c.submit(slowinc, 1, delay=0.5, workers=a.address)
-    while not a.executing_count:
-        await asyncio.sleep(0.01)
-
-    steal.move_task_request(
-        s.tasks[future.key], s.workers[a.address], s.workers[b.address]
-    )
-    await asyncio.sleep(0.1)
-    assert a.tasks[future.key].state == "executing"
-    assert not b.executing_count
+    await asyncio.gather(*(w.close() for w in workers))
 
 
 @gen_cluster(client=True)
 async def test_dont_steal_already_released(c, s, a, b):
     future = c.submit(slowinc, 1, delay=0.05, workers=a.address)
     key = future.key
-    await asyncio.sleep(0.05)
-    assert key in a.tasks
+    while key not in a.tasks:
+        await asyncio.sleep(0.05)
+
     del future
-    await asyncio.sleep(0.05)
-    # In case the system is slow (e.g. network) ensure that nothing bad happens
-    # if the key was already released
-    assert key not in a.tasks
-    a.steal_request(key)
-    assert a.batched_stream.buffer == [
-        {"op": "steal-response", "key": key, "state": None}
-    ]
+
+    while key in a.tasks and a.tasks[key].state != "released":
+        await asyncio.sleep(0.05)
+
+    a.handle_steal_request(key=key, stimulus_id="test")
+    assert len(a.batched_stream.buffer) == 1
+    msg = a.batched_stream.buffer[0]
+    assert msg["op"] == "steal-response"
+    assert msg["key"] == key
+    assert msg["state"] in [None, "released"]
+
     with captured_logger(
         logging.getLogger("distributed.stealing"), level=logging.DEBUG
     ) as stealing_logs:
-        await asyncio.sleep(0.05)
-
-    logs = stealing_logs.getvalue()
-    assert f"Key released between request and confirm: {key}" in logs
+        msg = f"Key released between request and confirm: {key}"
+        while msg not in stealing_logs.getvalue():
+            await asyncio.sleep(0.05)
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
@@ -737,17 +899,15 @@ async def test_dont_steal_long_running_tasks(c, s, a, b):
         ) <= 1
 
 
-@pytest.mark.xfail(
-    sys.version_info[:2] == (3, 8),
-    reason="Sporadic failure on Python 3.8",
-    strict=False,
+@gen_cluster(
+    client=True,
+    nthreads=[("127.0.0.1", 5)] * 2,
+    config={"distributed.scheduler.work-stealing-interval": "20ms"},
 )
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 5)] * 2)
 async def test_cleanup_repeated_tasks(c, s, a, b):
     class Foo:
         pass
 
-    s.extensions["stealing"]._pc.callback_time = 20
     await c.submit(slowidentity, -1, delay=0.1)
     objects = [c.submit(Foo, pure=False, workers=a.address) for _ in range(50)]
 
@@ -794,21 +954,18 @@ async def test_lose_task(c, s, a, b):
     assert "Error" not in out
 
 
-@gen_cluster(client=True)
-async def test_worker_stealing_interval(c, s, a, b):
+@pytest.mark.parametrize("interval, expected", [(None, 100), ("500ms", 500), (2, 2)])
+@gen_cluster(nthreads=[])
+async def test_parse_stealing_interval(s, interval, expected):
     from distributed.scheduler import WorkStealing
 
-    ws = WorkStealing(s)
-    assert ws._pc.callback_time == 100
-
-    with dask.config.set({"distributed.scheduler.work-stealing-interval": "500ms"}):
+    if interval:
+        ctx = dask.config.set({"distributed.scheduler.work-stealing-interval": "500ms"})
+    else:
+        ctx = contextlib.nullcontext()
+    with ctx:
         ws = WorkStealing(s)
-    assert ws._pc.callback_time == 500
-
-    # Default unit is `ms`
-    with dask.config.set({"distributed.scheduler.work-stealing-interval": 2}):
-        ws = WorkStealing(s)
-    assert ws._pc.callback_time == 2
+        s.periodic_callbacks["stealing"].callback_time == expected
 
 
 @gen_cluster(client=True)
@@ -821,7 +978,250 @@ async def test_balance_with_longer_task(c, s, a, b):
         slowinc, 1, delay=5, workers=[a.address], priority=1
     )  # a surprisingly long task
     z = c.submit(
-        inc, x, workers=[a.address], allow_other_workers=True, priority=0
+        slowadd, x, 1, workers=[a.address], allow_other_workers=True, priority=0
     )  # a task after y, suggesting a, but open to b
+
+    # Allow task to be learned, otherwise it will not be stolen
+    _ = c.submit(slowadd, x, 2, workers=[b.address])
     await z
+    assert not y.done()
     assert z.key in b.data
+
+
+@gen_cluster(client=True)
+async def test_blocklist_shuffle_split(c, s, a, b):
+
+    pd = pytest.importorskip("pandas")
+    dd = pytest.importorskip("dask.dataframe")
+    npart = 10
+    df = dd.from_pandas(pd.DataFrame({"A": range(100), "B": 1}), npartitions=npart)
+    graph = df.shuffle(
+        "A",
+        shuffle="tasks",
+        # If we don't have enough partitions, we'll fall back to a simple shuffle
+        max_branch=npart - 1,
+    ).sum()
+    res = c.compute(graph)
+
+    while not s.tasks:
+        await asyncio.sleep(0.005)
+    prefixes = set(s.task_prefixes.keys())
+    from distributed.stealing import fast_tasks
+
+    blocked = fast_tasks & prefixes
+    assert blocked
+    assert any(["split" in prefix for prefix in blocked])
+
+    stealable = s.extensions["stealing"].stealable
+    while not res.done():
+        for tasks_per_level in stealable.values():
+            for tasks in tasks_per_level:
+                for ts in tasks:
+                    assert ts.prefix.name not in fast_tasks
+                    assert "split" not in ts.prefix.name
+        await asyncio.sleep(0.001)
+    await res
+
+
+@gen_cluster(
+    client=True,
+    nthreads=[("", 1)] * 3,
+    config={
+        "distributed.scheduler.work-stealing-interval": 1_000_000,
+    },
+)
+async def test_steal_concurrent_simple(c, s, *workers):
+    steal = s.extensions["stealing"]
+    w0 = workers[0]
+    w1 = workers[1]
+    w2 = workers[2]
+    futs1 = c.map(
+        slowinc,
+        range(10),
+        key=[f"f1-{ix}" for ix in range(10)],
+        workers=[w0.address],
+    )
+
+    while not w0.tasks:
+        await asyncio.sleep(0.1)
+
+    # ready is a heap but we don't need last, just not the next
+    _, victim_key = w0.ready[-1]
+
+    ws0 = s.workers[w0.address]
+    ws1 = s.workers[w1.address]
+    ws2 = s.workers[w2.address]
+    victim_ts = s.tasks[victim_key]
+    steal.move_task_request(victim_ts, ws0, ws1)
+    steal.move_task_request(victim_ts, ws0, ws2)
+
+    await c.gather(futs1)
+
+    # First wins
+    assert ws1.has_what
+    assert not ws2.has_what
+
+
+@gen_cluster(
+    client=True,
+    config={
+        "distributed.scheduler.work-stealing-interval": 1_000_000,
+    },
+)
+async def test_steal_reschedule_reset_in_flight_occupancy(c, s, *workers):
+    # https://github.com/dask/distributed/issues/5370
+    steal = s.extensions["stealing"]
+    w0 = workers[0]
+    futs1 = c.map(
+        slowinc,
+        range(10),
+        key=[f"f1-{ix}" for ix in range(10)],
+    )
+    while not w0.tasks:
+        await asyncio.sleep(0.01)
+
+    # ready is a heap but we don't need last, just not the next
+    _, victim_key = w0.ready[-1]
+
+    victim_ts = s.tasks[victim_key]
+
+    wsA = victim_ts.processing_on
+    other_workers = [ws for ws in s.workers.values() if ws != wsA]
+    wsB = other_workers[0]
+
+    steal.move_task_request(victim_ts, wsA, wsB)
+
+    s.reschedule(victim_key)
+    await c.gather(futs1)
+
+    del futs1
+
+    assert all(v == 0 for v in steal.in_flight_occupancy.values())
+
+
+@gen_cluster(
+    client=True,
+    config={
+        "distributed.scheduler.work-stealing-interval": 10,
+    },
+)
+async def test_get_story(c, s, *workers):
+    steal = s.extensions["stealing"]
+    futs = c.map(
+        slowinc, range(100), workers=[workers[0].address], allow_other_workers=True
+    )
+    collect = c.submit(sum, futs)
+    await collect
+    key = next(iter(workers[1].tasks))
+    ts = s.tasks[key]
+    msgs = steal.story(key)
+    msgs_ts = steal.story(ts)
+    assert msgs
+    assert msgs == msgs_ts
+    assert all(isinstance(m, tuple) for m in msgs)
+
+
+@gen_cluster(
+    client=True,
+    nthreads=[("", 1)] * 3,
+    config={
+        "distributed.scheduler.work-stealing-interval": 1_000_000,
+    },
+)
+async def test_reschedule_concurrent_requests_deadlock(c, s, *workers):
+    # https://github.com/dask/distributed/issues/5370
+    steal = s.extensions["stealing"]
+    w0 = workers[0]
+    futs1 = c.map(
+        slowinc,
+        range(10),
+        key=[f"f1-{ix}" for ix in range(10)],
+        workers=[w0.address],
+        allow_other_workers=True,
+    )
+    while not w0.active_keys:
+        await asyncio.sleep(0.01)
+
+    # ready is a heap but we don't need last, just not the next
+    victim_key = list(w0.active_keys)[0]
+
+    victim_ts = s.tasks[victim_key]
+
+    wsA = victim_ts.processing_on
+    other_workers = [ws for ws in s.workers.values() if ws != wsA]
+    wsB = other_workers[0]
+    wsC = other_workers[1]
+
+    steal.move_task_request(victim_ts, wsA, wsB)
+
+    s.set_restrictions(worker={victim_key: [wsB.address]})
+    s.reschedule(victim_key)
+    assert wsB == victim_ts.processing_on
+    # move_task_request is not responsible for respecting worker restrictions
+    steal.move_task_request(victim_ts, wsB, wsC)
+    await c.gather(futs1)
+
+    # If this turns out to be overly flaky, the following may be relaxed or
+    # removed. The point of this test is to not deadlock but verifying expected
+    # state is still a nice thing
+
+    # Either the last request goes through or both have been rejected since the
+    # computation was already done by the time the request comes in. This is
+    # unfortunately not stable even if we increase the compute time
+    if victim_ts.who_has != {wsC}:
+        msgs = steal.story(victim_ts)
+        assert len(msgs) == 2
+        assert all(msg[0] == "already-aborted" for msg in msgs), msgs
+
+
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 3)
+async def test_correct_bad_time_estimate(c, s, *workers):
+    """Initial time estimation causes the task to not be considered for
+    stealing. Following occupancy readjustments will re-enlist the keys since
+    the duration estimate is now significant.
+
+    This is done during reevaluate occupancy
+    """
+    steal = s.extensions["stealing"]
+    future = c.submit(slowinc, 1, delay=0)
+    await wait(future)
+    futures = [c.submit(slowinc, future, delay=0.1, pure=False) for i in range(20)]
+    while not any(f.key in s.tasks for f in futures):
+        await asyncio.sleep(0.001)
+    assert not any(s.tasks[f.key] in steal.key_stealable for f in futures)
+    await asyncio.sleep(0.5)
+    assert any(s.tasks[f.key] in steal.key_stealable for f in futures)
+    await wait(futures)
+    assert all(w.data for w in workers), [sorted(w.data) for w in workers]
+
+
+@gen_cluster(client=True)
+async def test_steal_stimulus_id_unique(c, s, *workers):
+    steal = s.extensions["stealing"]
+    num_futs = 1_000
+    async with Lock() as lock:
+
+        def blocked(x, lock):
+            lock.acquire()
+
+        # Setup all tasks on worker 0 such that victim/thief relation is the
+        # same for all tasks.
+        futures = c.map(
+            blocked, range(num_futs), lock=lock, workers=[workers[0].address]
+        )
+        # Ensure all tasks are assigned to the worker since otherwise the
+        # move_task_request fails.
+        while len(workers[0].tasks) != num_futs:
+            await asyncio.sleep(0.1)
+        tasks = [s.tasks[f.key] for f in futures]
+        w0 = s.workers[workers[0].address]
+        w1 = s.workers[workers[1].address]
+        # Generating the move task requests as fast as possible increases the
+        # chance of duplicates if the uniqueness is not guaranteed.
+        for ts in tasks:
+            steal.move_task_request(ts, w0, w1)
+        # Values stored in in_flight are used for response verification.
+        # Therefore all stimulus IDs are stored here and must be unique
+        stimulus_ids = {dct["stimulus_id"] for dct in steal.in_flight.values()}
+        assert len(stimulus_ids) == num_futs
+        await c.cancel(futures)
