@@ -1,35 +1,30 @@
-from concurrent.futures import CancelledError
-from operator import add
-import random
-import sys
-from time import sleep
 import asyncio
+import random
+from contextlib import suppress
+from operator import add
+from time import sleep
+
+import pytest
+from tlz import concat, sliding_window
 
 from dask import delayed
-import pytest
-from toolz import concat, sliding_window
 
-from distributed import Client, wait, Nanny
+from distributed import Client, Nanny, wait
+from distributed.compatibility import WINDOWS
 from distributed.config import config
 from distributed.metrics import time
-from distributed.utils import All, ignoring
+from distributed.utils import CancelledError
 from distributed.utils_test import (
-    gen_cluster,
-    cluster,
-    inc,
-    slowinc,
-    slowadd,
-    slowsum,
     bump_rlimit,
-)
-from distributed.utils_test import (  # noqa: F401
-    loop,
+    cluster,
+    gen_cluster,
+    inc,
     nodebug_setup_module,
     nodebug_teardown_module,
+    slowadd,
+    slowinc,
+    slowsum,
 )
-from distributed.client import wait
-from tornado import gen
-
 
 # All tests here are slow in some way
 setup_module = nodebug_setup_module
@@ -37,17 +32,18 @@ teardown_module = nodebug_teardown_module
 
 
 @gen_cluster(client=True)
-def test_stress_1(c, s, a, b):
+async def test_stress_1(c, s, a, b):
     n = 2 ** 6
 
     seq = c.map(inc, range(n))
     while len(seq) > 1:
-        yield gen.sleep(0.1)
+        await asyncio.sleep(0.1)
         seq = [c.submit(add, seq[i], seq[i + 1]) for i in range(0, len(seq), 2)]
-    result = yield seq[0]
+    result = await seq[0]
     assert result == sum(map(inc, range(n)))
 
 
+@pytest.mark.slow
 @pytest.mark.parametrize(("func", "n"), [(slowinc, 100), (inc, 1000)])
 def test_stress_gc(loop, func, n):
     with cluster() as (s, [a, b]):
@@ -59,22 +55,20 @@ def test_stress_gc(loop, func, n):
             assert x.result() == n + 2
 
 
-@pytest.mark.skipif(
-    sys.platform.startswith("win"), reason="test can leave dangling RPC objects"
-)
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 8, timeout=None)
-def test_cancel_stress(c, s, *workers):
+@pytest.mark.skipif(WINDOWS, reason="test can leave dangling RPC objects")
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 8)
+async def test_cancel_stress(c, s, *workers):
     da = pytest.importorskip("dask.array")
     x = da.random.random((50, 50), chunks=(2, 2))
     x = c.persist(x)
-    yield wait([x])
+    await wait([x])
     y = (x.sum(axis=0) + x.sum(axis=1) + 1).std()
     n_todo = len(y.dask) - len(x.dask)
     for i in range(5):
         f = c.compute(y)
         while len(s.waiting) > (random.random() + 1) * 0.5 * n_todo:
-            yield gen.sleep(0.01)
-        yield c._cancel(f)
+            await asyncio.sleep(0.01)
+        await c._cancel(f)
 
 
 def test_cancel_stress_sync(loop):
@@ -91,39 +85,46 @@ def test_cancel_stress_sync(loop):
                 c.cancel(f)
 
 
-@gen_cluster(nthreads=[], client=True, timeout=None)
-def test_stress_creation_and_deletion(c, s):
+@pytest.mark.xfail(
+    reason="Flaky and re-fails on rerun. See https://github.com/dask/distributed/issues/5388"
+)
+@pytest.mark.slow
+@gen_cluster(
+    nthreads=[],
+    client=True,
+    timeout=180,
+    scheduler_kwargs={"allowed_failures": 100_000},
+)
+async def test_stress_creation_and_deletion(c, s):
     # Assertions are handled by the validate mechanism in the scheduler
-    s.allowed_failures = 100000
     da = pytest.importorskip("dask.array")
 
-    x = da.random.random(size=(2000, 2000), chunks=(100, 100))
-    y = (x + 1).T + (x * 2) - x.mean(axis=1)
-
+    rng = da.random.RandomState(0)
+    x = rng.random(size=(2000, 2000), chunks=(100, 100))
+    y = ((x + 1).T + (x * 2) - x.mean(axis=1)).sum().round(2)
     z = c.persist(y)
 
-    @gen.coroutine
-    def create_and_destroy_worker(delay):
+    async def create_and_destroy_worker(delay):
         start = time()
         while time() < start + 5:
-            n = yield Nanny(s.address, nthreads=2, loop=s.loop)
-            yield gen.sleep(delay)
-            yield n.close()
+            async with Nanny(s.address, nthreads=2) as n:
+                await asyncio.sleep(delay)
             print("Killed nanny")
 
-    yield asyncio.wait_for(
-        All([create_and_destroy_worker(0.1 * i) for i in range(20)]), 60
-    )
+    await asyncio.gather(*(create_and_destroy_worker(0.1 * i) for i in range(20)))
+
+    async with Nanny(s.address, nthreads=2):
+        assert await c.compute(z) == 8000884.93
 
 
 @gen_cluster(nthreads=[("127.0.0.1", 1)] * 10, client=True, timeout=60)
-def test_stress_scatter_death(c, s, *workers):
+async def test_stress_scatter_death(c, s, *workers):
     import random
 
     s.allowed_failures = 1000
     np = pytest.importorskip("numpy")
-    L = yield c.scatter([np.random.random(10000) for i in range(len(workers))])
-    yield c.replicate(L, n=2)
+    L = await c.scatter([np.random.random(10000) for i in range(len(workers))])
+    await c.replicate(L, n=2)
 
     adds = [
         delayed(slowadd, pure=True)(
@@ -148,7 +149,7 @@ def test_stress_scatter_death(c, s, *workers):
     from distributed.scheduler import logger
 
     for i in range(7):
-        yield gen.sleep(0.1)
+        await asyncio.sleep(0.1)
         try:
             s.validate_state()
         except Exception as c:
@@ -160,11 +161,11 @@ def test_stress_scatter_death(c, s, *workers):
             else:
                 raise
         w = random.choice(alive)
-        yield w.close()
+        await w.close()
         alive.remove(w)
 
-    with ignoring(CancelledError):
-        yield c.gather(futures)
+    with suppress(CancelledError):
+        await c.gather(futures)
 
     futures = None
 
@@ -173,11 +174,13 @@ def vsum(*args):
     return sum(args)
 
 
-@pytest.mark.avoid_travis
+@pytest.mark.avoid_ci
 @pytest.mark.slow
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 80, timeout=1000)
-def test_stress_communication(c, s, *workers):
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 80)
+async def test_stress_communication(c, s, *workers):
     s.validate = False  # very slow otherwise
+    for w in workers:
+        w.validate = False
     da = pytest.importorskip("dask.array")
     # Test consumes many file descriptors and can hang if the limit is too low
     resource = pytest.importorskip("resource")
@@ -186,17 +189,17 @@ def test_stress_communication(c, s, *workers):
     n = 20
     xs = [da.random.random((100, 100), chunks=(5, 5)) for i in range(n)]
     ys = [x + x.T for x in xs]
-    z = da.atop(vsum, "ij", *concat(zip(ys, ["ij"] * n)), dtype="float64")
+    z = da.blockwise(vsum, "ij", *concat(zip(ys, ["ij"] * n)), dtype="float64")
 
     future = c.compute(z.sum())
 
-    result = yield future
+    result = await future
     assert isinstance(result, float)
 
 
 @pytest.mark.skip
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 10, timeout=60)
-def test_stress_steal(c, s, *workers):
+async def test_stress_steal(c, s, *workers):
     s.validate = False
     for w in workers:
         w.validate = False
@@ -210,7 +213,7 @@ def test_stress_steal(c, s, *workers):
     future = c.compute(total)
 
     while future.status != "finished":
-        yield gen.sleep(0.1)
+        await asyncio.sleep(0.1)
         for i in range(3):
             a = random.choice(workers)
             b = random.choice(workers)
@@ -221,8 +224,8 @@ def test_stress_steal(c, s, *workers):
 
 
 @pytest.mark.slow
-@gen_cluster(nthreads=[("127.0.0.1", 1)] * 10, client=True, timeout=120)
-def test_close_connections(c, s, *workers):
+@gen_cluster(nthreads=[("127.0.0.1", 1)] * 10, client=True, timeout=180)
+async def test_close_connections(c, s, *workers):
     da = pytest.importorskip("dask.array")
     x = da.random.random(size=(1000, 1000), chunks=(1000, 1))
     for i in range(3):
@@ -231,7 +234,7 @@ def test_close_connections(c, s, *workers):
 
     future = c.compute(x.sum())
     while any(s.processing.values()):
-        yield gen.sleep(0.5)
+        await asyncio.sleep(0.5)
         worker = random.choice(list(workers))
         for comm in worker._comms:
             comm.abort()
@@ -239,33 +242,37 @@ def test_close_connections(c, s, *workers):
         # for w in workers:
         #     print(w)
 
-    yield wait(future)
+    await wait(future)
 
 
+@pytest.mark.slow
 @pytest.mark.xfail(
-    reason="IOStream._handle_write blocks on large write_buffer"
-    " https://github.com/tornadoweb/tornado/issues/2110"
+    reason="flaky and re-fails on rerun; "
+    "IOStream._handle_write blocks on large write_buffer"
+    " https://github.com/tornadoweb/tornado/issues/2110",
 )
-@gen_cluster(client=True, timeout=20, nthreads=[("127.0.0.1", 1)])
-def test_no_delay_during_large_transfer(c, s, w):
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)])
+async def test_no_delay_during_large_transfer(c, s, w):
     pytest.importorskip("crick")
     np = pytest.importorskip("numpy")
     x = np.random.random(100000000)
     x_nbytes = x.nbytes
 
     # Reset digests
-    from distributed.counter import Digest
     from collections import defaultdict
     from functools import partial
+
     from dask.diagnostics import ResourceProfiler
 
+    from distributed.counter import Digest
+
     for server in [s, w]:
         server.digests = defaultdict(partial(Digest, loop=server.io_loop))
         server._last_tick = time()
 
     with ResourceProfiler(dt=0.01) as rprof:
-        future = yield c.scatter(x, direct=True, hash=False)
-        yield gen.sleep(0.5)
+        future = await c.scatter(x, direct=True, hash=False)
+        await asyncio.sleep(0.5)
 
     rprof.close()
     x = None  # lose ref
