@@ -1,8 +1,8 @@
-from time import sleep
 import threading
+from time import sleep
 
 from distributed.metrics import time
-from distributed.threadpoolexecutor import ThreadPoolExecutor, secede, rejoin
+from distributed.threadpoolexecutor import ThreadPoolExecutor, rejoin, secede
 
 
 def test_tpe():
@@ -62,11 +62,16 @@ def test_shutdown_wait():
 
 def test_secede_rejoin_busy():
     with ThreadPoolExecutor(2) as e:
+        # Prime threads
+        f1 = e.submit(sleep, 0.1)
+        f2 = e.submit(sleep, 0.1)
+        f1.result()
+        f2.result()
 
         def f():
             assert threading.current_thread() in e._threads
             secede()
-            sleep(0.1)
+            sleep(0.2)
             assert threading.current_thread() not in e._threads
             rejoin()
             assert len(e._threads) == 2
@@ -74,12 +79,13 @@ def test_secede_rejoin_busy():
             return threading.current_thread()
 
         future = e.submit(f)
-        L = [e.submit(sleep, 0.2) for i in range(10)]
+        for _ in range(6):
+            e.submit(sleep, 0.4)
         start = time()
         special_thread = future.result()
         stop = time()
 
-        assert 0.1 < stop - start < 0.3
+        assert 0.2 < stop - start < 0.6
 
         assert len(e._threads) == 2
         assert special_thread in e._threads
